@@ -83,6 +83,7 @@ enum
 
 enum {
   PROP_0,
+  PROP_PROFILE,
   PROP_ICON_TITLE,
   PROP_ICON_TITLE_SET,
   PROP_OVERRIDE_COMMAND,
@@ -103,17 +104,20 @@ static void terminal_screen_init        (TerminalScreen      *screen);
 static void terminal_screen_class_init  (TerminalScreenClass *klass);
 static void terminal_screen_dispose     (GObject             *object);
 static void terminal_screen_finalize    (GObject             *object);
-static void terminal_screen_update_on_realize (VteTerminal *vte_terminal,
-                                               TerminalScreen *screen);
+static void terminal_screen_drag_data_received (GtkWidget        *widget,
+                                                GdkDragContext   *context,
+                                                gint              x,
+                                                gint              y,
+                                                GtkSelectionData *selection_data,
+                                                guint             info,
+                                                guint             time);
 static void terminal_screen_system_font_notify_cb (TerminalApp *app,
                                                    GParamSpec *pspec,
                                                    TerminalScreen *screen);
 static void terminal_screen_change_font (TerminalScreen *screen);
-static gboolean terminal_screen_popup_menu         (GtkWidget      *term,
-                                                    TerminalScreen *screen);
-static gboolean terminal_screen_button_press_event (GtkWidget      *term,
-                                                    GdkEventButton *event,
-                                                    TerminalScreen *screen);
+static gboolean terminal_screen_popup_menu (GtkWidget *widget);
+static gboolean terminal_screen_button_press (GtkWidget *widget,
+                                              GdkEventButton *event);
 
 static void terminal_screen_window_title_changed      (VteTerminal *vte_terminal,
                                                        TerminalScreen *screen);
@@ -132,15 +136,6 @@ static void terminal_screen_cook_icon_title (TerminalScreen *screen);
 
 static void queue_recheck_working_dir (TerminalScreen *screen);
 
-static void drag_data_received (TerminalScreen   *widget,
-                                GdkDragContext   *context,
-                                gint              x,
-                                gint              y,
-                                GtkSelectionData *selection_data,
-                                guint             info,
-                                guint             time,
-                                gpointer          data);
-
 static void  terminal_screen_match_add         (TerminalScreen            *screen,
                                                 const char           *regexp,
                                                 int                   flavor);
@@ -167,15 +162,6 @@ free_tag_data (TagData *tagdata)
   g_slice_free (TagData, tagdata);
 }
 
-static void
-style_set_callback (GtkWidget *widget,
-                    GtkStyle  *previous_style,
-                    void      *data)
-{
-  if (GTK_WIDGET_REALIZED (widget))
-    terminal_screen_change_font (TERMINAL_SCREEN (data));
-}
-
 #ifdef DEBUG_GEOMETRY
 static void
 parent_size_request (GtkWidget *scrolled_window, GtkRequisition *req, GtkWidget *screen)
@@ -286,6 +272,43 @@ terminal_screen_screen_changed (GtkWidget *widget, GdkScreen *previous_screen)
                     G_CALLBACK (terminal_screen_sync_settings), widget);
 }
 
+static void
+terminal_screen_realize (GtkWidget *widget)
+{
+  TerminalScreen *screen = TERMINAL_SCREEN (widget);
+  TerminalScreenPrivate *priv = screen->priv;
+  TerminalBackgroundType bg_type;
+
+  GTK_WIDGET_CLASS (terminal_screen_parent_class)->realize (widget);
+
+  g_assert (priv->window != NULL);
+
+  /* FIXME: Don't enable this if we have a compmgr. */
+  bg_type = terminal_profile_get_property_enum (priv->profile, TERMINAL_PROFILE_BACKGROUND_TYPE);
+  vte_terminal_set_background_transparent (VTE_TERMINAL (screen),
+                                           bg_type == TERMINAL_BACKGROUND_TRANSPARENT &&
+                                           !terminal_window_uses_argb_visual (priv->window));
+
+  /* FIXME: why do this on realize? */
+  terminal_window_set_size (priv->window, screen, TRUE);
+}
+
+static void
+terminal_screen_style_set (GtkWidget *widget,
+                           GtkStyle *previous_style)
+{
+  TerminalScreen *screen = TERMINAL_SCREEN (widget);
+  void (* style_set) (GtkWidget*, GtkStyle*) = GTK_WIDGET_CLASS (terminal_screen_parent_class)->style_set;
+
+  if (style_set)
+    style_set (widget, previous_style);
+
+  update_color_scheme (screen);
+
+  if (GTK_WIDGET_REALIZED (widget))
+    terminal_screen_change_font (screen);
+}
+
 #ifdef DEBUG_GEOMETRY
 
 static void size_request (GtkWidget *widget, GtkRequisition *req)
@@ -383,30 +406,11 @@ terminal_screen_init (TerminalScreen *screen)
   gtk_target_table_free (targets, n_targets);
   gtk_target_list_unref (target_list);
 
-  g_signal_connect (screen, "drag_data_received",
-                    G_CALLBACK (drag_data_received),
-                    screen);
-
-  g_signal_connect (screen,
-                    "realize",
-                    G_CALLBACK (terminal_screen_update_on_realize),
-                    screen);
-
-  g_signal_connect (screen,
-                    "style_set",
-                    G_CALLBACK (style_set_callback),
-                    screen);
-
   g_signal_connect (screen,
                     "popup_menu",
                     G_CALLBACK (terminal_screen_popup_menu),
                     screen);
 
-  g_signal_connect (screen,
-                    "button_press_event",
-                    G_CALLBACK (terminal_screen_button_press_event),
-                    screen);
-
   priv->title_from_arg = NULL;
   priv->user_title = FALSE;
   
@@ -442,6 +446,9 @@ terminal_screen_get_property (GObject *object,
 
   switch (prop_id)
     {
+      case PROP_PROFILE:
+        g_value_set_object (value, terminal_screen_get_profile (screen));
+        break;
       case PROP_ICON_TITLE:
         g_value_set_string (value, terminal_screen_get_icon_title (screen));
         break;
@@ -470,6 +477,14 @@ terminal_screen_set_property (GObject *object,
 
   switch (prop_id)
     {
+      case PROP_PROFILE: {
+        TerminalProfile *profile;
+
+        profile = g_value_get_object (value);
+        g_assert (profile != NULL);
+        terminal_screen_set_profile (screen, profile);
+        break;
+      }
       case PROP_OVERRIDE_COMMAND:
         terminal_screen_set_override_command (screen, g_value_get_boxed (value));
         break;
@@ -495,6 +510,11 @@ terminal_screen_class_init (TerminalScreenClass *klass)
   object_class->set_property = terminal_screen_set_property;
 
   widget_class->screen_changed = terminal_screen_screen_changed;
+  widget_class->realize = terminal_screen_realize;
+  widget_class->style_set = terminal_screen_style_set;
+  widget_class->drag_data_received = terminal_screen_drag_data_received;
+  widget_class->button_press_event = terminal_screen_button_press;
+  widget_class->popup_menu = terminal_screen_popup_menu;
 
   signals[PROFILE_SET] =
     g_signal_new (I_("profile-set"),
@@ -546,7 +566,14 @@ terminal_screen_class_init (TerminalScreenClass *klass)
                   g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE,
                   0);
-  
+
+  g_object_class_install_property
+    (object_class,
+     PROP_PROFILE,
+     g_param_spec_string ("profile", NULL, NULL,
+                          NULL,
+                          G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+
   g_object_class_install_property
     (object_class,
      PROP_ICON_TITLE,
@@ -799,7 +826,6 @@ terminal_screen_profile_notify_cb (TerminalProfile *profile,
         }
       
       /* FIXME: Don't enable this if we have a compmgr. */
-      /* FIXME: redo this on screen-changed! */
       vte_terminal_set_background_transparent (vte_terminal,
                                                bg_type == TERMINAL_BACKGROUND_TRANSPARENT &&
                                                (!priv->window || !terminal_window_uses_argb_visual (priv->window)));
@@ -848,10 +874,7 @@ cook_title  (TerminalScreen *screen, const char *raw_title, char **old_cooked_ti
 
   g_return_val_if_fail (old_cooked_title != NULL, FALSE);
 
-  if (priv->profile)
-    mode = terminal_profile_get_property_enum (priv->profile, TERMINAL_PROFILE_TITLE_MODE);
-  else
-    mode = TERMINAL_TITLE_REPLACE;
+  mode = terminal_profile_get_property_enum (priv->profile, TERMINAL_PROFILE_TITLE_MODE);
 
   /* use --title argument if one was supplied, otherwise ask the profile */
   if (priv->title_from_arg)
@@ -1007,9 +1030,6 @@ terminal_screen_system_font_notify_cb (TerminalApp *app,
 {
   TerminalScreenPrivate *priv = screen->priv;
 
-  if (!priv->profile)
-    return;
-
   if (!GTK_WIDGET_REALIZED (screen))
     return;
 
@@ -1020,22 +1040,13 @@ terminal_screen_system_font_notify_cb (TerminalApp *app,
 }
 
 static void
-terminal_screen_update_on_realize (VteTerminal *vte_terminal,
-                                   TerminalScreen *screen)
+terminal_screen_change_font (TerminalScreen *screen)
 {
   TerminalScreenPrivate *priv = screen->priv;
 
-  update_color_scheme (screen);
-
-  /* FIXME: why do this on realize? */
-  terminal_window_set_size (priv->window, screen, TRUE);
-}
-
-static void
-terminal_screen_change_font (TerminalScreen *screen)
-{
   terminal_screen_set_font (screen);
-  terminal_screen_update_on_realize (VTE_TERMINAL (screen), screen);
+
+  terminal_window_set_size (priv->window, screen, TRUE);
 }
 
 static void
@@ -1045,7 +1056,7 @@ profile_forgotten_callback (TerminalProfile *profile,
   TerminalProfile *new_profile;
 
   new_profile = terminal_app_get_profile_for_new_term (terminal_app_get ());
-  g_assert (new_profile);
+  g_assert (new_profile != NULL);
   terminal_screen_set_profile (screen, new_profile);
 }
 
@@ -1095,12 +1106,17 @@ terminal_screen_set_profile (TerminalScreen *screen,
 
   if (old_profile)
     g_object_unref (old_profile);
+
+  g_object_notify (G_OBJECT (screen), "profile");
 }
 
 TerminalProfile*
 terminal_screen_get_profile (TerminalScreen *screen)
 {
-  return screen->priv->profile;
+  TerminalScreenPrivate *priv = screen->priv;
+
+  g_assert (priv->profile != NULL);
+  return priv->profile;
 }
 
 void
@@ -1472,9 +1488,9 @@ terminal_screen_popup_info_unref (TerminalScreenPopupInfo *info)
 }
 
 static gboolean
-terminal_screen_popup_menu (GtkWidget      *term,
-                            TerminalScreen *screen)
+terminal_screen_popup_menu (GtkWidget *widget)
 {
+  TerminalScreen *screen = TERMINAL_SCREEN (widget);
   TerminalScreenPopupInfo *info;
 
   info = terminal_screen_popup_info_new (screen);
@@ -1488,10 +1504,10 @@ terminal_screen_popup_menu (GtkWidget      *term,
 }
 
 static gboolean
-terminal_screen_button_press_event (GtkWidget      *widget,
-                                    GdkEventButton *event,
-                                    TerminalScreen *screen)
+terminal_screen_button_press (GtkWidget      *widget,
+                              GdkEventButton *event)
 {
+  TerminalScreen *screen = TERMINAL_SCREEN (widget);
   TerminalScreenPrivate *priv = screen->priv;
   int char_width, char_height;
   gboolean dingus_button;
@@ -1560,6 +1576,9 @@ terminal_screen_button_press_event (GtkWidget      *widget,
     }
 
   /* default behavior is to let the terminal widget deal with it */
+  if (GTK_WIDGET_CLASS (terminal_screen_parent_class)->button_press_event)
+    return GTK_WIDGET_CLASS (terminal_screen_parent_class)->button_press_event (widget, event);
+
   return FALSE;
 }
 
@@ -1796,9 +1815,7 @@ terminal_screen_widget_child_died (GtkWidget      *term,
 
   priv->child_pid = -1;
   
-  action = TERMINAL_EXIT_CLOSE;
-  if (priv->profile)
-    action = terminal_profile_get_property_enum (priv->profile, TERMINAL_PROFILE_EXIT_ACTION);
+  action = terminal_profile_get_property_enum (priv->profile, TERMINAL_PROFILE_EXIT_ACTION);
   
   switch (action)
     {
@@ -1832,16 +1849,16 @@ terminal_screen_set_user_title (TerminalScreen *screen,
 }
 
 static void
-drag_data_received (TerminalScreen   *widget,
-                    GdkDragContext   *context,
-                    gint              x,
-                    gint              y,
-                    GtkSelectionData *selection_data,
-                    guint             info,
-                    guint             time,
-                    gpointer          data)
+terminal_screen_drag_data_received (GtkWidget        *widget,
+                                    GdkDragContext   *context,
+                                    gint              x,
+                                    gint              y,
+                                    GtkSelectionData *selection_data,
+                                    guint             info,
+                                    guint             time)
 {
-  TerminalScreen *screen = TERMINAL_SCREEN (data);
+  TerminalScreen *screen = TERMINAL_SCREEN (widget);
+  TerminalScreenPrivate *priv = screen->priv;
 
 #if 0
   {
@@ -1894,7 +1911,6 @@ drag_data_received (TerminalScreen   *widget,
       {
         guint16 *data = (guint16 *)selection_data->data;
         GdkColor color;
-        TerminalProfile *profile;
 
         /* We accept drops with the wrong format, since the KDE color
          * chooser incorrectly drops application/x-color with format 8.
@@ -1907,16 +1923,11 @@ drag_data_received (TerminalScreen   *widget,
         color.blue = data[2];
         /* FIXME: use opacity from data[3] */
 
-        profile = terminal_screen_get_profile (screen);
-
-        if (profile)
-          {
-            g_object_set (profile,
-                          TERMINAL_PROFILE_BACKGROUND_TYPE, TERMINAL_BACKGROUND_SOLID,
-                          TERMINAL_PROFILE_USE_THEME_COLORS, FALSE,
-                          TERMINAL_PROFILE_BACKGROUND_COLOR, &color,
-                          NULL);
-          }
+        g_object_set (priv->profile,
+                      TERMINAL_PROFILE_BACKGROUND_TYPE, TERMINAL_BACKGROUND_SOLID,
+                      TERMINAL_PROFILE_USE_THEME_COLORS, FALSE,
+                      TERMINAL_PROFILE_BACKGROUND_COLOR, &color,
+                      NULL);
       }
       break;
 
@@ -1995,14 +2006,12 @@ drag_data_received (TerminalScreen   *widget,
 
         if (uris && uris[0])
           {
-            TerminalProfile *profile;
             char *filename;
 
-            profile = terminal_screen_get_profile (screen);
             filename = g_filename_from_uri (uris[0], NULL, NULL);
-            if (filename && profile)
+            if (filename)
               {
-                g_object_set (profile,
+                g_object_set (priv->profile,
                               TERMINAL_PROFILE_BACKGROUND_TYPE, TERMINAL_BACKGROUND_IMAGE,
                               TERMINAL_PROFILE_BACKGROUND_IMAGE_FILE, filename,
                               NULL);
@@ -2016,18 +2025,9 @@ drag_data_received (TerminalScreen   *widget,
       break;
 
     case TARGET_RESET_BG:
-      {
-        TerminalProfile *profile;
-
-        profile = terminal_screen_get_profile (screen);
-        
-        if (profile)
-          {
-                g_object_set (profile,
-                              TERMINAL_PROFILE_BACKGROUND_TYPE, TERMINAL_BACKGROUND_SOLID,
-                              NULL);
-          }
-      }
+      g_object_set (priv->profile,
+                    TERMINAL_PROFILE_BACKGROUND_TYPE, TERMINAL_BACKGROUND_SOLID,
+                    NULL);
       break;
 
     case TARGET_TAB:
@@ -2067,20 +2067,16 @@ drag_data_received (TerminalScreen   *widget,
 void
 _terminal_screen_update_scrollbar (TerminalScreen *screen)
 {
-  TerminalProfile *profile;
+  TerminalScreenPrivate *priv = screen->priv;
   GtkWidget *parent;
   GtkPolicyType policy = GTK_POLICY_ALWAYS;
   GtkCornerType corner = GTK_CORNER_TOP_LEFT;
 
-  profile = terminal_screen_get_profile (screen);
-  if (profile == NULL)
-    return;
-
   parent = GTK_WIDGET (screen)->parent;
   if (!parent)
     return;
 
-  switch (terminal_profile_get_property_enum (profile, TERMINAL_PROFILE_SCROLLBAR_POSITION))
+  switch (terminal_profile_get_property_enum (priv->profile, TERMINAL_PROFILE_SCROLLBAR_POSITION))
     {
     case TERMINAL_SCROLLBAR_HIDDEN:
       policy = GTK_POLICY_NEVER;
