@@ -72,6 +72,7 @@
 static gboolean initialization_complete = FALSE;
 static GSList *pending_new_terminal_events = NULL;
 static BonoboListener *listener = NULL;
+static gboolean factory_registered = FALSE;
 
 typedef struct
 {
@@ -88,7 +89,6 @@ typedef struct
   char   **post_execute_args;
 
   gboolean  execute;
-  char     *geometry;
   gboolean  use_factory;
   char     *zoom;
 } OptionParsingResults;
@@ -187,19 +187,14 @@ static void
 apply_defaults (OptionParsingResults *results,
                 InitialWindow        *iw)
 {
-  g_assert (iw->geometry == NULL);
-
   if (results->default_role)
     {
       iw->role = results->default_role;
       results->default_role = NULL;
     }
   
-  if (results->default_geometry)
-    {
-      iw->geometry = results->default_geometry;
-      results->default_geometry = NULL;
-    }
+  if (iw->geometry == NULL)
+    iw->geometry = g_strdup (results->default_geometry);
 
   if (results->default_window_menubar_forced)
     {
@@ -320,9 +315,8 @@ option_window_callback (const gchar *option_name,
                         GError     **error)
 {
   OptionParsingResults *results = data;
-  InitialWindow *iw;
 
-  iw = add_new_window (results, NULL, FALSE);
+  add_new_window (results, NULL, FALSE);
 
   return TRUE;
 }
@@ -526,8 +520,7 @@ option_hide_menubar_callback (const gchar *option_name,
   return TRUE;
 }
 
-
-static gboolean 
+static gboolean
 option_fullscreen_callback (const gchar *option_name,
                             const gchar *value,
                             gpointer     data,
@@ -547,8 +540,28 @@ option_fullscreen_callback (const gchar *option_name,
   return TRUE;
 }
 
+static gboolean
+option_geometry_callback (const gchar *option_name,
+                          const gchar *value,
+                          gpointer     data,
+                          GError     **error)
+{
+  OptionParsingResults *results = data;
 
-static gboolean 
+  if (results->initial_windows)
+    {
+      InitialWindow *iw;
+
+      iw = g_list_last (results->initial_windows)->data;
+      iw->geometry = g_strdup (value);
+    }
+  else
+    results->default_geometry = g_strdup (value);
+
+  return TRUE;
+}
+
+static gboolean
 option_disable_factory_callback (const gchar *option_name,
                                  const gchar *value,
                                  gpointer     data,
@@ -598,10 +611,9 @@ option_active_callback (const gchar *option_name,
                         gpointer     data,
                         GError     **error)
 {
-  OptionParsingResults *results;
+  OptionParsingResults *results = data;
   InitialTab *it;
 
-  results = data;
   it = ensure_top_tab (results);
   it->active = TRUE;
 
@@ -616,11 +628,8 @@ digest_options_callback (GOptionContext *context,
                          gpointer      data,
                          GError      **error)
 {
-  OptionParsingResults *results;
+  OptionParsingResults *results = data;
   InitialTab    *it;
-  InitialWindow *iw;
-
-  results = data;
 
   /* make sure we have some window in case no options were given */
   if (results->initial_windows == NULL)
@@ -643,19 +652,6 @@ digest_options_callback (GOptionContext *context,
       it->exec_argv = results->post_execute_args;
       results->post_execute_args = NULL;
     } 
-         
-
-  if (results->geometry)
-    {
-      if (results->initial_windows)
-        {
-          iw = g_list_last (results->initial_windows)->data;
-          iw->geometry = g_strdup (results->geometry);
-        }
-      else if (!results->default_geometry)
-        results->default_geometry = g_strdup (results->geometry);
-    }
-
 
   if (results->zoom)
     {
@@ -723,7 +719,6 @@ option_parsing_results_new (int *argc, char **argv)
   results->initial_windows = NULL;
   results->default_role = NULL;
   results->default_geometry = NULL;
-  results->geometry = NULL;
   results->zoom = NULL;
 
   results->screen_number = -1;
@@ -781,7 +776,6 @@ option_parsing_results_free (OptionParsingResults *results)
 
   g_free (results->display_name);
   g_free (results->startup_id);
-  g_free (results->geometry);
   g_free (results->zoom);
 
   g_slice_free (OptionParsingResults, results);
@@ -1236,8 +1230,10 @@ main (int argc, char **argv)
 
   terminal_app_shutdown ();
 
-  if (listener)
+  if (factory_registered)
     bonobo_activation_active_server_unregister (ACT_IID, BONOBO_OBJREF (listener));
+  if (listener)
+    bonobo_object_unref (BONOBO_OBJECT (listener));
 
   g_object_unref (program);
 
@@ -1371,8 +1367,8 @@ get_goption_context (OptionParsingResults *parsing_results)
       "geometry",
       0,
       0,
-      G_OPTION_ARG_STRING,
-      &parsing_results->geometry,
+      G_OPTION_ARG_CALLBACK,
+      option_geometry_callback,
       N_("X geometry specification (see \"X\" man page), can be specified once per window to be opened."),
       N_("GEOMETRY")
     },
@@ -1397,10 +1393,10 @@ get_goption_context (OptionParsingResults *parsing_results)
     {
       "startup-id",
       0,
-      0,
+      G_OPTION_FLAG_HIDDEN,
       G_OPTION_ARG_STRING,
       &parsing_results->startup_id,
-      N_("ID for startup notification protocol."),
+      NULL,
       NULL
     },
     {
@@ -1446,7 +1442,7 @@ get_goption_context (OptionParsingResults *parsing_results)
       G_OPTION_ARG_CALLBACK,
       option_active_callback,
       N_("Set the last specified tab as the active one in its window"),
-      N_("ZOOMFACTOR")
+      NULL
     },
     
     /*
@@ -1756,8 +1752,14 @@ terminal_register_as_factory (void)
   result = bonobo_activation_active_server_register (
     per_display_iid, BONOBO_OBJREF (listener));
 
-  if (result != Bonobo_ACTIVATION_REG_SUCCESS)
-    bonobo_object_unref (BONOBO_OBJECT (listener));
+  factory_registered = (result == Bonobo_ACTIVATION_REG_SUCCESS);
+
+  if (!factory_registered)
+    {
+      bonobo_object_unref (BONOBO_OBJECT (listener));
+      listener = NULL;
+    }
+
 
 #ifdef DEBUG_FACTORY
   if (result == Bonobo_ACTIVATION_REG_SUCCESS)
@@ -1772,7 +1774,7 @@ terminal_register_as_factory (void)
 static gboolean
 terminal_invoke_factory (int argc, char **argv)
 {
-  Bonobo_Listener listener;
+  Bonobo_Listener receiver;
 
   switch (terminal_register_as_factory ())
     {
@@ -1793,10 +1795,10 @@ terminal_invoke_factory (int argc, char **argv)
         break;
     }
 
-  listener = bonobo_activation_activate_from_id (
+  receiver = bonobo_activation_activate_from_id (
     ACT_IID, Bonobo_ACTIVATION_FLAG_EXISTING_ONLY, NULL, NULL);
 
-  if (listener != CORBA_OBJECT_NIL)
+  if (receiver != CORBA_OBJECT_NIL)
     {
       int i;
       CORBA_any any;
@@ -1814,8 +1816,8 @@ terminal_invoke_factory (int argc, char **argv)
       for (i = 0; i < args._length; i++)
         args._buffer [i] = argv [i];
       
-      Bonobo_Listener_event (listener, "new_terminal", &any, &ev);
-      CORBA_Object_release (listener, &ev);
+      Bonobo_Listener_event (receiver, "new_terminal", &any, &ev);
+      CORBA_Object_release (receiver, &ev);
       if (!BONOBO_EX (&ev))
         return TRUE;
 
